from cpl_core.configuration import ConfigurationABC
from cpl_discord.command import DiscordCommandABC
from cpl_discord.service import DiscordBotServiceABC
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.logging.command_logger import CommandLogger
from modules.base.configuration.base_server_settings import BaseServerSettings


class HelpCommand(DiscordCommandABC):

    def __init__(
            self,
            config: ConfigurationABC,
            logger: CommandLogger,
            message_service: MessageServiceABC,
            bot: DiscordBotServiceABC,
            client_utils: ClientUtilsServiceABC
    ):
        DiscordCommandABC.__init__(self)

        self._config = config
        self._logger = logger
        self._message_service = message_service
        self._bot = bot
        self._client_utils = client_utils

        self._logger.trace(__name__, f'Loaded command service: {type(self).__name__}')

    @commands.hybrid_command()
    @commands.guild_only()
    async def help(self, ctx: Context, persistent_flag: str = None):
        self._logger.debug(__name__, f'Received command help {ctx}:{persistent_flag}')
        if not await self._client_utils.check_if_bot_is_ready_yet_and_respond(ctx):
            return
        self._client_utils.received_command(ctx.guild.id)
        settings: BaseServerSettings = self._config.get_configuration(f'BaseServerSettings_{ctx.guild.id}')
        is_persistent = persistent_flag == '--stay'
        await self._message_service.send_ctx_msg(ctx, settings.help_command_reference_url, is_persistent=is_persistent)
        self._logger.trace(__name__, f'Finished help command')
